/*
 * mount.c	Mount a remote directory
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include "nfsbench.h"
#include "fs/mount.h"
#include "fs/portmap.h"
#include "xdr/mount.h"

struct context {
	u32		rpcstat;
	u32		mountstat;
	struct nfs_fh *	fh;
};

static unsigned short	mount_port = 0;

static void
mount_mnt_reply(struct nfsreq *req, u32 *buffer, int len)
{
	struct context	*ctxt;

	ctxt = (struct context *) req->rq_clntdata;
	xdr_mount_mnt_res(buffer, &ctxt->mountstat, ctxt->fh);
	ctxt->rpcstat = rpcstat;
}

int
mount_mnt(char *path, struct nfsnode *vn)
{
	struct context	*ctxt;
	struct nfsreq	*req;
	sockaddr_in	server;
	u32		*msg;

	if (mount_port == 0) {
		if (!pmap_getport(MOUNT_PROGRAM, MOUNT_VERSION, &mount_port)) {
			script_error("portmapper failure: "
				     "couldn't get mount port\n");
			return 0;
		}
	}

	ctxt = (struct context *) xmalloc(sizeof(*ctxt));
	ctxt->fh = &vn->n_fh;

	if (!(req = generate_request())) {
		biod_flush();
		req = generate_request();
	}

	server.sin_family = AF_INET;
	server.sin_addr = nfs_addr;
	server.sin_port = mount_port;

	req->rq_addr = &server;
	msg = xdr_mount_mnt(req->rq_msg, path);
	req->rq_msglen = (msg - req->rq_msg) << 2;
	req->rq_callback = mount_mnt_reply;
	req->rq_clntdata = ctxt;

	enqueue_request(req);
	biod_flush();

	if (ctxt->rpcstat != RPC_SUCCESS) {
		script_error("mount call failed, RPC error %d\n",
			ctxt->rpcstat);
		xfree(ctxt);
		return 0;
	}
	if (ctxt->mountstat != 0) {
		script_error("mount call failed; access denied\n");
		xfree(ctxt);
		return 0;
	}

	vn->n_flags |= N_VALIDFH;
	xfree(ctxt);
	return 1;
}
