/*
 * portmap.c
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include "nfsbench.h"
#include "fs/portmap.h"
#include "xdr/portmap.h"

struct context {
	struct pmap	pmap;
	u32		rpcstat;
};

static void
pmap_getport_reply(struct nfsreq *req, u32 *buffer, int len)
{
	struct context	*ctxt;

	ctxt = (struct context *) req->rq_clntdata;
	xdr_pmap_getport_res(buffer, &ctxt->pmap.pm_port);
	ctxt->rpcstat = rpcstat;
}

int
pmap_getport(u32 prog, u32 vers, unsigned short *portp)
{
	struct context	*ctxt;
	struct nfsreq	*req;
	sockaddr_in	server;
	u32		*msg;

	ctxt = (struct context *) xmalloc(sizeof(*ctxt));
	ctxt->pmap.pm_prog = prog;
	ctxt->pmap.pm_vers = vers;
	ctxt->pmap.pm_prot = IPPROTO_UDP;
	ctxt->pmap.pm_port = 0;

	server.sin_family = AF_INET;
	server.sin_addr = nfs_addr;
	server.sin_port = htons(PMAP_PORT);

	if (!(req = generate_request())) {
		biod_flush();
		req = generate_request();
	}

	req->rq_addr = &server;
	msg = xdr_pmap_getport(req->rq_msg, &ctxt->pmap);
	req->rq_msglen = (msg - req->rq_msg) << 2;
	req->rq_callback = pmap_getport_reply;
	req->rq_clntdata = ctxt;

	enqueue_request(req);
	biod_flush();

	if (ctxt->rpcstat != RPC_SUCCESS) {
		script_error("getport call failed, RPC error %d\n",
			ctxt->rpcstat);
		xfree(ctxt);
		return 0;
	}

	*portp = ntohs((unsigned short) ctxt->pmap.pm_port);
	xfree(ctxt);
	return 1;
}
