/*
 * nfsbench.c	Central functions for nfsbench
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include "nfsbench.h"
#include "xgetopt.h"
#include "xdr/nfs.h"

/*
 * Configuration variables.
 */
int			nrbiods = 4;
int			nrsocks = 1;
int			nfs_rsize = 4096;
int			nfs_wsize = 4096;
char *			nfs_root = "/tmp/nfsbench";
struct in_addr		nfs_addr;
unsigned short		nfs_port;
int			nfs_udp = 1;
sockaddr_in		nfs_server;

/*
 * Option table.
 */
struct xoption		longopts[] = {
 { "biods",	'b',	OPT_INT,	NULL,	&nrbiods,
					"Number of biod `processes'"	},
 { "help",	'h',	OPT_HELP,	NULL,	NULL,
					"Print this help message"	},
 { "port",	'p',	OPT_PORT,	NULL,	&nfs_port,
					"NFS server port (2049)"	},
 { "root",	'd',	OPT_STRING,	NULL,	&nfs_root,
					"Mount point (/tmp/nfsbench)"	},
 { "rsize",	'r',	OPT_INT,	NULL,	&nfs_rsize,
					"Read packet size (4096)"	},
 { "server",	's',	OPT_ADDR,	NULL,	&nfs_addr,
					"NFS server address (127.0.0.1)"},
 { "sockets",	'S',	OPT_INT,	NULL,	&nrsocks,
					"Number of biod sockets (1)"	},
 { "tcp",	'T',	OPT_FALSE,	NULL,	&nfs_udp,
					"NFS over TCP (n/a)"		},
 { "udp",	'U',	OPT_TRUE,	NULL,	&nfs_udp,
					"NFS over UDP (default)"	},
 { "wsize",	'w',	OPT_INT,	NULL,	&nfs_wsize,
					"Write packet size (4096)"	},

 { NULL,	'\0',	OPT_END,	NULL,	NULL,
					NULL				}
};

int
main(int argc, char **argv)
{
	int	idx;

	if (sizeof(u32) != 4) {
		fprintf(stderr, "sizeof(u32) = %d instead of 4; please "
				"adjust the typedef in %s and recompile\n",
				sizeof(u32), __FILE__);
		exit(1);
	}

	nfs_addr.s_addr = htonl(INADDR_LOOPBACK);
	nfs_port = htons(NFS_PORT);

	if (!xgetopt(argc, argv, longopts, &idx))
		return 1;

	nfs_server.sin_family = AF_INET;
	nfs_server.sin_addr = nfs_addr;
	nfs_server.sin_port = nfs_port;

	while (idx < argc) {
		int		arg;

		arg = idx;
		while (arg < argc && strcmp(argv[arg], "--")) 
			arg++;
		if (script_run(argv[idx], arg - idx, argv + idx) != AR_OK) {
			printf("Failed..\n");
		}
		idx = arg + 1;
	}

	return 0;
}
