/*
 * nfsbench.h	Common definitions for nfsbench
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#ifndef NFSBENCH_H
#define NFSBENCH_H

#include <sys/types.h>
#include <netinet/in.h>
#include "config.h"

#if SIZEOF_UNSIGNED_LONG == 4
# define u32		unsigned long
#elif SIZEOF_UNSIGNED_INT == 4
# define u32		unsigned int
#else
# error "Don't know how to define unsigned 32-bit type"
#endif

typedef struct sockaddr_in	sockaddr_in;

#define MAXRWSIZE	(16 * 1024)
#define MAXPKTSIZE	(MAXRWSIZE + 1024)
#define MAXBIODS	32
#define MAXREQUESTS	(2 * MAXBIODS)
#define MAXREXMITS	16
#define INITIMEO	(6 * HZ / 10)		/* 0.6 seconds */
#define DEFAULT_UID	5555
#define DEFAULT_GID	5555

struct nfsreq;
typedef void	(*rqcallback_t)(struct nfsreq *req, u32 *buf, int len);

struct nfsreq {
	struct nfsreq *	rq_next;
	sockaddr_in *	rq_addr;
	int		rq_sock;
	u32		rq_xid;
	u32 *		rq_msg;
	unsigned int	rq_msglen;
	unsigned long	rq_qtime;
	unsigned long	rq_rtt;
	unsigned long	rq_expires;
	unsigned long	rq_timeout;
	unsigned short	rq_rexmits;
	rqcallback_t	rq_callback;
	void *		rq_clntdata;
};

struct nfsstat {
	int			pkt_sent;
	int			pkt_recv;
	int			pkt_lost;
	int			pkt_rxmt;
	int			byte_sent;
	int			byte_recv;

	unsigned long		start_time;
	unsigned long		total_time;
	unsigned long		rtt;
	unsigned long		min_rtt;
	unsigned long		max_rtt;
};

extern struct nfsreq *	req_pending;
extern struct nfsreq *	req_pending_tail;
extern struct nfsreq *	req_queued;
extern struct nfsreq *	req_queued_tail;
extern struct nfsreq *	req_free;
extern struct nfsstat	stats;

/*
 * Configuration variables.
 */
extern int		nrbiods;
extern int		nrsocks;
extern int		nfs_rsize;
extern int		nfs_wsize;
extern char *		nfs_root;
extern sockaddr_in	nfs_server;
extern struct in_addr	nfs_addr;
extern unsigned short	nfs_port;

#include "script.h"
#include "xdr/rpc.h"
#include "xdr/nfs.h"

/* Approximation of current time in jiffies */
extern unsigned long	xtime;

/*
 * Functions
 */
void		biod(void);
void		biod_drain(void);
void		biod_drain_on(void);
void		biod_flush(void);
struct nfsreq *	generate_request(void);
void		enqueue_request(struct nfsreq *);
void		insert_request(struct nfsreq **, struct nfsreq *);
void		finish_request(struct nfsreq *, u32 *, int);
unsigned long	jiffies(void);
char *		jiffies_print(unsigned long time);
char *		jiffies_print_100(unsigned long time);
int		addr_parse(char *hname, struct in_addr *ap);
char *		addr_print(struct in_addr *ap);
int		fh_parse(char *, struct nfs_fh *);
char *		fh_print(struct nfs_fh *);
u32		rpcsvc_parse(char *name);
void *		xmalloc_fl(unsigned long, char *, int);
char *		xstrdup(char *sp);
void		xfree(void *);
void		clear_stats(void);
void		print_stats(void);

#define append_request(head, tail, req)	\
	req->rq_next = NULL;					\
	if (!tail) {						\
		head = tail = req;				\
	} else if (tail->rq_expires < (req)->rq_expires) {	\
		tail->rq_next = req;				\
		tail = req;					\
	} else {						\
		insert_request(&head, req);			\
		if (!(req)->rq_next)				\
			tail = (req);				\
	}
#define wait_request(req)		\
	append_request(req_pending, req_pending_tail, req)
#define xmalloc(size)		xmalloc_fl(size, __FILE__, __LINE__)
#define biod_sync(statement)		\
	{							\
		biod_flush();					\
		statement;					\
		biod_flush();					\
	}

#endif /* NFSBENCH_H */
