/*
 * engine.c	General purpose `script' evaluation engine.
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#include "nfsbench.h"

#define MAXSCRSTACK	1024

struct regs {
	struct script *	st_script;
	action_fn_t *	st_actions;
	void *		st_context;
	int		st_argc;
	char **		st_argv;
	int		st_len;
	int		st_pc;
};

static struct regs *	cur = NULL;
static struct regs	stack[MAXSCRSTACK];
static int		sp = 0;
static int		calling = 0;
static int		status;
static char *		result;

struct script *
script_find(char *name)
{
	struct script	*script;
	int		i;

	for (i = 0; (script = test_table[i]) != NULL; i++) {
		if (!strcmp(script->sc_name, name))
			return script;
	}

	return NULL;
}

int
script_call(char *name, int argc, char **argv)
{
	struct script	*script;
	struct regs	*sub;

	if (!(script = script_find(name))) {
		script_error("can't find script %s\n", name);
		return AR_ABORT;
	}
	sub = &stack[sp];
	sub->st_actions = script->sc_actions;
	sub->st_script = script;
	sub->st_argc = argc;
	sub->st_argv = argv;
	sub->st_len = script->sc_len;
	sub->st_pc = 0;

	if (result)
		free(result);
	status = AR_OK;
	result = NULL;

	calling = 1;

	biod();
	biod_drain();

	if (script->sc_finish)
		script->sc_finish(sub->st_context);
	cur = &stack[--sp-1];

	return status;
}

int
script_callv(char *name, ...)
{
	va_list		ap;
	char		*argv[16];
	int		argc = 0;

	argv[argc++] = name;
	va_start(ap, name);
	do {
		argv[argc++] = va_arg(ap, char *);
		if (argc >= 16 && argv[argc-1] != NULL) {
			fprintf(stderr, "too many arguments in call to %s\n",
					name);
			return AR_ABORT;
		}
	} while (argv[argc-1] != NULL);
	va_end(ap);

	return script_call(name, argc - 1, argv);
}

int
script_run(char *name, int argc, char **argv)
{
	if (sp) {
		fprintf(stderr, "There's already a script running, can't "
				"execute %s\n", name);
		return AR_ABORT;
	}

	printf("Evaluating script %s\n", name);
	return script_call(name, argc, argv);
}

int
script_nextaction(void)
{
	action_fn_t	ac;

	if (calling) {
		struct script	*script;
		struct regs	*sub;

		calling = 0;
		sub = &stack[sp++];
		script = sub->st_script;
		if (script->sc_initialize) {
			sub->st_context = script->sc_initialize(sub->st_argc,
						sub->st_argv);
			if (!sub->st_context) {
				sp--;
				return 0;
			}
		}
		cur = sub;
		return 1;	/* call again */
	}
	if (!cur || cur->st_pc >= cur->st_len)
		return 0;

	ac = cur->st_actions[cur->st_pc];
	switch ((*ac)(cur->st_context)) {
	case AR_DRAIN:
		biod_drain_on();
	case AR_OK:
		cur->st_pc++;
		break;
	case AR_AGAIN:
		break;
	/*
	case AR_DONE:
		cur->st_pc = cur->st_len;
		break;
	 */
	case AR_ABORT:
		fprintf(stderr, "Aborting program.\n");
		exit (2);
	}

	return 1;
}

void
script_error(char *fmt, ...)
{
	va_list		ap;

	fflush(stdout);
	if (cur)
		fprintf(stderr, "%s: ", cur->st_script->sc_name);
	else
		fprintf(stderr, "error: ");

	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	fflush(stderr);
}

void
script_setresult(char *value)
{
	if (result)
		free(result);
	result = strdup(value);
}

char *
script_getresult(void)
{
	return result;
}
