/*
 * stats.c
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */
#include <sys/param.h>
#include <memory.h>
#include <stdio.h>
#include "nfsbench.h"
#include "fs/nfs.h"

struct nfsstat	stats;

void
clear_stats(void)
{
	memset(&stats, 0, sizeof(stats));
	stats.start_time = jiffies();
	stats.min_rtt = ~0;
}

void
print_stats(void)
{
	unsigned long	bps = 0, rtt = 0;

	stats.total_time = jiffies() - stats.start_time;
	if (stats.total_time) {
		if (stats.byte_recv > stats.byte_sent)
			bps = stats.byte_recv * 8 * HZ / stats.total_time;
		else
			bps = stats.byte_sent * 8 * HZ / stats.total_time;
	}
	if (stats.pkt_recv)
		rtt = 100 * stats.rtt / stats.pkt_recv;
	if (stats.min_rtt == ~0)
		stats.min_rtt = 0;

	printf("\ttotal duration:       %s ms\n"
	       "\tbytes sent:           %d\n"
	       "\tbytes received:       %d\n"
	       "\tpackets sent:         %d\n"
	       "\tpackets reveived:     %d\n"
	       "\tpackets retransmits:  %d\n"
	       "\tpackets lost:         %d\n"
	       "\tavg. rtt:             %s ms\n"
	       "\tmin. rtt:             %s ms\n"
	       "\tmax. rtt:             %s ms\n"
	       "\tbandwidth:            %ld kbit/s %ld kByte/s\n",
		jiffies_print(stats.total_time),
		stats.byte_sent,
		stats.byte_recv,
		stats.pkt_sent,
		stats.pkt_recv,
		stats.pkt_rxmt,
		stats.pkt_lost,
		jiffies_print_100(rtt),
		jiffies_print(stats.min_rtt),
		jiffies_print(stats.max_rtt),
		bps / 1000, bps / 8000);
	return;
}
