/*
 * test0.c	Try to gauge nfsping thruput
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/param.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "nfsbench.h"
#include "xgetopt.h"
#include "xdr/nfs.h"
#include "fs/nfs.h"

struct ping_context;

static void *	ping_init(int argc, char **argv);
static int	ping(struct ping_context *);
static void	ping_finish(struct ping_context *);

static action_fn_t	actions[] = {
	(action_fn_t) ping,
};

struct script	nfsping_script = {
	"nfsping",
	(scinit_fn_t) ping_init,
	(scdone_fn_t) ping_finish,
	nrof(actions),
	actions
};

struct ping_context {
	u32		packets;
	u32		payload;
	struct in_addr	addr;
	int		port;
	int		sent;
};

static int
ping(struct ping_context *ctxt)
{
	while (nfs_null(ctxt->payload)) {
		if (++(ctxt->sent) >= ctxt->packets)
			return AR_OK;
	}

	return AR_AGAIN;
}

/*
 * Options understood by ping script
 */
static int	nrpackets;
static int	payload;
static struct xoption	longopts[] = {
 { "payload",	'b',	OPT_INT,	NULL,	&payload,
					"Packet payload",	},
 { "help",	'h',	OPT_HELP,	NULL,	NULL,
					"Print this message"	},
 { "packets",	'n',	OPT_INT,	NULL,	&nrpackets,
					"Number of packets",	},

 { NULL,	0,	OPT_END,	NULL,	NULL,	NULL	}
};

/*
 * Initialize ping script
 */
static void *
ping_init(int argc, char **argv)
{
	struct ping_context	*ctxt;
	int			idx;

	ctxt = (struct ping_context *) xmalloc(sizeof(*ctxt));
	nrpackets = 1024;
	payload = 1024;

	if (!xgetopt(argc, argv, longopts, &idx))
		return 0;

	ctxt->packets = nrpackets;
	ctxt->payload = payload;

	clear_stats();

	return ctxt;
}

static void
ping_finish(struct ping_context *ctxt)
{
	print_stats();
	xfree(ctxt);
}
