/*
 * nfsread.c	Try to gauge nfs READ thruput
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/param.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "nfsbench.h"
#include "xgetopt.h"
#include "fs/nfs.h"

struct context;

static int	read(struct context *);
static void *	read_init(int argc, char **argv);
static void	read_finish(struct context *);

static action_fn_t	actions[] = {
	(action_fn_t) read,
};

struct script	nfsread_script = {
	"nfsread",
	(scinit_fn_t) read_init,
	(scdone_fn_t) read_finish,
	nrof(actions),
	actions
};

struct context {
	struct nfsnode *vnode;
	u32		offset;
	u32		endoffset;
};

static int
read(struct context *ctxt)
{
	ctxt->vnode->n_count++;
	while (nfs_read(ctxt->vnode, ctxt->offset)) {
		if ((ctxt->offset += nfs_rsize) >= ctxt->endoffset)
			return AR_OK;
		ctxt->vnode->n_count++;
	}
	return AR_AGAIN;
}

/*
 * Options understood by read
 */
static char *	path;
static int	nrpackets;
static int	nrbytes;
static int	offset;

static struct xoption	longopts[] = {
 { "offset",	'o',	OPT_INT,	NULL,	&offset,
					"Read offset"		},
 { "packets",	'p',	OPT_INT,	NULL,	&nrpackets,
					"Number of packets"	},
 { "bytes",	'n',	OPT_INT,	NULL,	&nrbytes,
					"Number of bytes"	},
 { "file",	'f',	OPT_STRING,	NULL,	&path,
					"Name of file"		},
 { "help",	'h',	OPT_HELP,	NULL,	NULL,
					"Print this message"	},

 { NULL,	0,	OPT_END,	NULL,	NULL,	NULL	}
};

static void *
read_init(int argc, char **argv)
{
	struct context	*ctxt;
	struct nfsnode	*vn;
	int		idx;

	nrpackets = 0;
	nrbytes = 0;
	offset = 0;
	path = "xxx";

	if (!xgetopt(argc, argv, longopts, &idx))
		return NULL;

	if (!(vn = nfs_open(path, 0)))
		return NULL;

	if (!nrbytes) {
		if (!nrpackets)
			nrpackets = 1024;
		nrbytes = nrpackets * nfs_rsize;
	}

	ctxt = (struct context *) xmalloc(sizeof(*ctxt));
	ctxt->vnode = vn;
	ctxt->offset = offset;
	ctxt->endoffset = offset + nrbytes;

	clear_stats();

	return ctxt;
}

static void
read_finish(struct context *ctxt)
{
	if (ctxt) {
		nfs_close(ctxt->vnode);
		print_stats();
		xfree(ctxt);
	}
}

