/*
 * util.c	Utility functions
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <ctype.h>

#include "nfsbench.h"

unsigned long	xtime = 0;

/*
 * Get current time in jiffies (units of 1/HZ seconds) since program start.
 */
unsigned long
jiffies(void)
{
	static unsigned long	start_time = 0;
	struct timeval		tv;

	if (!start_time)
		start_time = time(NULL);

	gettimeofday(&tv, NULL);
	if (!xtime)
		srand(tv.tv_usec);

	tv.tv_sec -= start_time;
	xtime = tv.tv_sec * HZ + tv.tv_usec * HZ / 1000000;
	return xtime;
}

/* This function interprets time as hundreths of a jiffy */
char *
jiffies_print_100(unsigned long time)
{
	static char	buffer[5][30];
	static int	nr = 0;
	unsigned long	msec100;
	int		i = nr;

	/* Convert time to 1/100 msec */
	msec100 = time * 1000 / HZ;
	sprintf(buffer[i], "%ld.%02ld", msec100 / 100, msec100 % 100);
	nr = (nr + 1) % 5;
	return buffer[i];
}

char *
jiffies_print(unsigned long time)
{
	return jiffies_print_100(100 * time);
}

int
addr_parse(char *hname, struct in_addr *ap)
{
	struct hostent	*hp;

	if (!(hp = gethostbyname(hname)))
		return 0;
	if (hp->h_addrtype != AF_INET)
		return 0;
	*ap = *(struct in_addr *) hp->h_addr;
	return 1;
}

char *
addr_print(struct in_addr *ap)
{
	return inet_ntoa(*ap);
}

int
fh_parse(char *fhname, struct nfs_fh *fh)
{
	unsigned char	*data;

	if (strlen(fhname) != 64)
		return 0;
	data = (unsigned char *) fh;
	while (*fhname != '\0') {
		*data++ = (tolower(fhname[0]) << 4) | tolower(fhname[1]);
		fhname += 2;
	}
	return 1;
}

char *
fh_print(struct nfs_fh *fh)
{
	static char	buffer[65];
	unsigned char	*data = (unsigned char *) fh;

	sprintf(buffer, "%02x%02x%02x%02x%02x%02x%02x%02x"
			"%02x%02x%02x%02x%02x%02x%02x%02x",
			data[0],  data[1],  data[2],  data[3],
			data[4],  data[5],  data[6],  data[7],
			data[8],  data[9],  data[10], data[11],
			data[12], data[13], data[14], data[15]);
	return buffer;
}

u32
rpcsvc_parse(char *name)
{
	struct rpcent	*rp;

	if (isdigit(name[0]))
		return atoi(name);
	if (!(rp = getrpcbyname(name)))
		return 0;
	return rp->r_number;
}

void *
xmalloc_fl(unsigned long size, char *file, int line)
{
	void	*ptr;

	if (!(ptr = malloc(size))) {
		fprintf(stderr, "malloc failed in %s:%d\nAbort.\n",
				file, line);
		exit (2);
	}

	memset(ptr, 0, size);
	return ptr;
}

void
xfree(void *ptr)
{
	free(ptr);
}

char *
xstrdup(char *sp)
{
	char	*res;

	res = (char *) xmalloc(strlen(sp) + 1);
	strcpy(res, sp);
	return res;
}
