/*
 * mount.c	XDR support for the NFS mount protocol version 1
 *		Only MNT/UMNT/UMNTALL supported; the rest would require
 *		TCP and I don't feel like that today.
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <string.h>
#include "nfsbench.h"
#include "xdr/nfs.h"
#include "xdr/mount.h"

#define ENC	1
#define DEC	0

/*
 * XDR encode/decode functions.
 */
static u32 *
xdr_nfs_fh(u32 *msg, int enc, nfs_fh *fh)
{
	if (enc)
		memcpy(msg, fh, sizeof(*fh));
	else
		memcpy(fh, msg, sizeof(*fh));
	msg += (sizeof(*fh) + 3) >> 2;
	return msg;
}

static u32 *
xdr_mount_fname(u32 *msg, int enc, char *fname)
{
	int	len;

	if (enc) {
		len = strlen(fname);
		*msg++ = htonl(len);
		strcpy((char *) msg, fname);
	} else {
		len = ntohl(*msg++);
		memcpy(fname, msg, len);
		fname[len] = 0;
	}
	msg += ((len + 3) >> 2);
	return msg;
}

static u32
xdr_mount_reply(u32 **msgp)
{
	rpcstat = rpc_reply(msgp);
	return rpcstat;
}

/*
 * MOUNT async procedures.
 */
u32 *
xdr_mount_null(u32 *msg)
{
	msg = rpc_header(msg, MOUNT_PROGRAM, MOUNT_VERSION, MOUNTPROC_NULL);
	return msg;
}

u32 *
xdr_mount_null_res(u32 *msg)
{
	xdr_mount_reply(&msg);
	return msg;
}

u32 *
xdr_mount_mnt(u32 *msg, char *path)
{
	msg = rpc_header(msg, MOUNT_PROGRAM, MOUNT_VERSION, MOUNTPROC_MNT);
	msg = xdr_mount_fname(msg, ENC, path);
	return msg;
}

u32 *
xdr_mount_mnt_res(u32 *msg, u32 *statp, nfs_fh *fh)
{
	if (xdr_mount_reply(&msg) != RPC_SUCCESS)
		return msg;
	if (!(*statp = ntohl(*msg++)))
		msg = xdr_nfs_fh(msg, DEC, fh);
	return msg;
}

u32 *
xdr_mount_umnt(u32 *msg, char *path)
{
	msg = rpc_header(msg, MOUNT_PROGRAM, MOUNT_VERSION, MOUNTPROC_UMNT);
	msg = xdr_mount_fname(msg, ENC, path);
	return msg;
}

u32 *
xdr_mount_umnt_res(u32 *msg)
{
	xdr_mount_reply(&msg);
	return msg;
}

u32 *
xdr_mount_umntall(u32 *msg)
{
	msg = rpc_header(msg, MOUNT_PROGRAM, MOUNT_VERSION, MOUNTPROC_UMNTALL);
	return msg;
}

u32 *
xdr_mount_umntall_res(u32 *msg)
{
	xdr_mount_reply(&msg);
	return msg;
}
