/*
 * nfs.h	Protocol definition for NFSv2.
 *
 * Originally generated by rpcgen, with heavy manual editing.
 */
#ifndef PROTO_NFS_H
#define PROTO_NFS_H

#include <sys/time.h>

#define NFS_PROGRAM		100003
#define NFS_VERSION		2
#define NFS_PORT		2049

#define NFS_MAXPATHLEN		1024
#define NFS_MAXNAMLEN		255
#define NFS_FHSIZE		32
#define NFS_COOKIESIZE		4
#define NFS_FIFO_DEV		-1
#define NFSMODE_FMT		0170000
#define NFSMODE_DIR		0040000
#define NFSMODE_CHR		0020000
#define NFSMODE_BLK		0060000
#define NFSMODE_REG		0100000
#define NFSMODE_LNK		0120000
#define NFSMODE_SOCK		0140000
#define NFSMODE_FIFO		0010000

#define NFSPROC_NULL		0
#define NFSPROC_GETATTR		1
#define NFSPROC_SETATTR		2
#define NFSPROC_ROOT		3
#define NFSPROC_LOOKUP		4
#define NFSPROC_READLINK	5
#define NFSPROC_READ		6
#define NFSPROC_WRITECACHE	7
#define NFSPROC_WRITE		8
#define NFSPROC_CREATE		9
#define NFSPROC_REMOVE		10
#define NFSPROC_RENAME		11
#define NFSPROC_LINK		12
#define NFSPROC_SYMLINK		13
#define NFSPROC_MKDIR		14
#define NFSPROC_RMDIR		15
#define NFSPROC_READDIR		16
#define NFSPROC_STATFS		17

enum {
	NFS_OK			= 0,
	NFSERR_PERM		= 1,
	NFSERR_NOENT		= 2,
	NFSERR_IO		= 5,
	NFSERR_NXIO		= 6,
	NFSERR_ACCES		= 13,
	NFSERR_EXIST		= 17,
	NFSERR_NODEV		= 19,
	NFSERR_NOTDIR		= 20,
	NFSERR_ISDIR		= 21,
	NFSERR_FBIG		= 27,
	NFSERR_NOSPC		= 28,
	NFSERR_ROFS		= 30,
	NFSERR_NAMETOOLONG	= 63,
	NFSERR_NOTEMPTY		= 66,
	NFSERR_DQUOT		= 69,
	NFSERR_STALE		= 70,
	NFSERR_WFLUSH		= 99,
};

enum {
	NFNON			= 0,
	NFREG			= 1,
	NFDIR			= 2,
	NFBLK			= 3,
	NFCHR			= 4,
	NFLNK			= 5,
	NFSOCK			= 6,
	NFBAD			= 7,
	NFFIFO			= 8,
};

typedef struct nfs_fh {
	char data[32];
} nfs_fh;

typedef struct nfs_fattr {
	u32		type;
	u32		mode;
	u32		nlink;
	u32		uid;
	u32		gid;
	u32		size;
	u32		blocksize;
	u32		rdev;
	u32		blocks;
	u32		fsid;
	u32		fileid;
	struct timeval	atime;
	struct timeval	mtime;
	struct timeval	ctime;
} nfs_fattr;

typedef struct nfs_sattr {
	u32		mode;
	u32		uid;
	u32		gid;
	u32		size;
	struct timeval	atime;
	struct timeval	mtime;
} nfs_sattr;

typedef struct nfs_dirent {
	u32		fileid;
	char *		name;
	u32		cookie;
	struct nfs_dirent *next;
} nfs_dirent;

typedef struct nfs_dirlist {
	nfs_dirent *	entries;
	u32		eof;
} nfs_dirlist;

typedef struct nfs_fstat {
	u32		tsize;
	u32		bsize;
	u32		blocks;
	u32		bfree;
	u32		bavail;
} nfs_fstat;

typedef void		(*nfs_dirent_fn_t)(void *, char *, u32);

extern int	nfsstat;

u32 *	xdr_nfs_null(u32 *);
u32 *	xdr_nfs_getattr(u32 *, nfs_fh *);
u32 *	xdr_nfs_getattr_res(u32 *, nfs_fattr *);
u32 *	xdr_nfs_setattr(u32 *, nfs_fh *, nfs_sattr *);
u32 *	xdr_nfs_setattr_res(u32 *, nfs_fattr *);
u32 *	xdr_nfs_root(u32 *);
u32 *	xdr_nfs_root_res(u32 *);
u32 *	xdr_nfs_lookup(u32 *, nfs_fh *, char *);
u32 *	xdr_nfs_lookup_res(u32 *, nfs_fh *, nfs_fattr *);
u32 *	xdr_nfs_readlink(u32 *, nfs_fh *);
u32 *	xdr_nfs_readlink_res(u32 *, char *);
u32 *	xdr_nfs_read(u32 *, nfs_fh *, u32, u32);
u32 *	xdr_nfs_read_res(u32 *, nfs_fattr *, u32 *);
u32 *	xdr_nfs_writecache(u32 *);
u32 *	xdr_nfs_writecache_res(u32 *);
u32 *	xdr_nfs_write(u32 *, nfs_fh *, u32, u32);
u32 *	xdr_nfs_write_res(u32 *, nfs_fattr *);
u32 *	xdr_nfs_create(u32 *, nfs_fh *, char *, nfs_sattr *);
u32 *	xdr_nfs_create_res(u32 *, nfs_fh *, nfs_fattr *);
u32 *	xdr_nfs_remove(u32 *, nfs_fh *, char *);
u32 *	xdr_nfs_remove_res(u32 *);
u32 *	xdr_nfs_rename(u32 *, nfs_fh *, char *, nfs_fh *, char *);
u32 *	xdr_nfs_rename_res(u32 *);
u32 *	xdr_nfs_link(u32 *, nfs_fh *, nfs_fh *, char *);
u32 *	xdr_nfs_link_res(u32 *);
u32 *	xdr_nfs_symlink(u32 *, nfs_fh *, char *, char *, nfs_sattr *);
u32 *	xdr_nfs_symlink_res(u32 *);
u32 *	xdr_nfs_mkdir(u32 *, nfs_fh *, char *, nfs_sattr *);
u32 *	xdr_nfs_mkdir_res(u32 *, nfs_fh *, nfs_fattr *);
u32 *	xdr_nfs_rmdir(u32 *, nfs_fh *, char *);
u32 *	xdr_nfs_rmdir_res(u32 *);
u32 *	xdr_nfs_readdir(u32 *, nfs_fh *, u32, u32);
u32 *	xdr_nfs_readdir_res(u32 *, nfs_dirent_fn_t, void *);
u32 *	xdr_nfs_statfs(u32 *, nfs_fh *);
u32 *	xdr_nfs_statfs_res(u32 *, nfs_fstat *);

char *	xdr_nfs_name(u32 op);

#endif /* PROTO_NFS_H */
