/*
 * portmap.c	XDR functions for the portmapper protocol
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include "nfsbench.h"
#include "xdr/portmap.h"

#define ENC	1
#define DEC	0

static u32 *
xdr_pmap(u32 *msg, int enc, struct pmap *pmap)
{
	if (enc) {
		*msg++ = htonl(pmap->pm_prog);
		*msg++ = htonl(pmap->pm_vers);
		*msg++ = htonl(pmap->pm_prot);
		*msg++ = htonl(pmap->pm_port);
	} else {
		pmap->pm_prog = ntohl(*msg++);
		pmap->pm_vers = ntohl(*msg++);
		pmap->pm_prot = ntohl(*msg++);
		pmap->pm_port = ntohl(*msg++);
	}
	return msg;
}

static u32
xdr_pmap_res(u32 **msgp)
{
	rpcstat = rpc_reply(msgp);
	return rpcstat;
}

/*
 * PMAP async procedures
 */
u32 *
xdr_pmap_null(u32 *msg)
{
	return rpc_header(msg, PMAP_PROGRAM, PMAP_VERSION, PMAPPROC_NULL);
}

u32 *
xdr_pmap_null_res(u32 *msg)
{
	xdr_pmap_res(&msg);
	return msg;
}

u32 *
xdr_pmap_set(u32 *msg, struct pmap *pmap)
{
	msg = rpc_header(msg, PMAP_PROGRAM, PMAP_VERSION, PMAPPROC_SET);
	msg = xdr_pmap(msg, ENC, pmap);
	return msg;
}

u32 *
xdr_pmap_set_res(u32 *msg, u32 *okay)
{
	if (xdr_pmap_res(&msg) != RPC_SUCCESS)
		return msg;
	*okay = ntohl(*msg++);
	return msg;
}

u32 *
xdr_pmap_unset(u32 *msg, struct pmap *pmap)
{
	msg = rpc_header(msg, PMAP_PROGRAM, PMAP_VERSION, PMAPPROC_UNSET);
	msg = xdr_pmap(msg, ENC, pmap);
	return msg;
}

u32 *
xdr_pmap_unset_res(u32 *msg, u32 *okay)
{
	if (xdr_pmap_res(&msg) != RPC_SUCCESS)
		return msg;
	*okay = ntohl(*msg++);
	return msg;
}

u32 *
xdr_pmap_getport(u32 *msg, struct pmap *pmap)
{
	msg = rpc_header(msg, PMAP_PROGRAM, PMAP_VERSION, PMAPPROC_GETPORT);
	msg = xdr_pmap(msg, ENC, pmap);
	return msg;
}

u32 *
xdr_pmap_getport_res(u32 *msg, u32 *port)
{
	if (xdr_pmap_res(&msg) != RPC_SUCCESS)
		return msg;
	*port = ntohl(*msg++);
	return msg;
}
