/*
 * rpc.c	RPC header generation, XDR, etc.
 *		RPC header generation is a major bottleneck.
 *		Improve by using pre-xdr'ed headers.
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include "nfsbench.h"

static u32	xid = 0;
u32		rpcstat;
int		auth_uid = DEFAULT_UID;
int		auth_gid = DEFAULT_GID;

u32 *
rpc_header(u32 *msg, u32 prog, u32 vers, u32 proc)
{
	if (!xid)
		xid = getpid() << 16;

	*msg++ = xid++;		/* XID */
	*msg++ = 0;		/* CALL */
	*msg++ = htonl(2);	/* RPC version */
	*msg++ = htonl(prog);
	*msg++ = htonl(vers);
	*msg++ = htonl(proc);
	*msg++ = htonl(1);	/* AUTH_UNIX */
	*msg++ = htonl(20);	/* cred len */
	*msg++ = htonl(jiffies() / HZ);
	*msg++ = 0;		/* no machine name */
	*msg++ = htonl(auth_uid);
	*msg++ = htonl(auth_gid);
	*msg++ = 0;		/* empty gid list */
	*msg++ = 0;		/* NULL verf */
	*msg++ = 0;		/* zero length verf */
	return msg;
}

u32
rpc_reply(u32 **msgp)
{
	u32	*msg = *msgp;
	u32	stat;
	int	len;

	msg++;				/* XID */
	if (ntohl(*msg++) != 1)		/* direction */
		return RPC_CANTDECODERES;
	if (ntohl(*msg++) == 0) {	/* reply_status */
		/* accepted */
		msg++;			/* skip verf flavor */
		len = ntohl(*msg++);	/* verf len */
		msg += (len + 3) >> 2;
		stat = ntohl(*msg++);	/* accept_status */
		*msgp = msg;
		switch (stat) {
		case 0: return RPC_SUCCESS;
		case 1: return RPC_PROGUNAVAIL;
		case 2: return RPC_PROGVERSMISMATCH;
		case 3: return RPC_PROCUNAVAIL;
		case 4: return RPC_CANTDECODEARGS;
		case 5: return RPC_SYSTEMERROR;
		default:return RPC_FAILED;
		}
	}
	/* rejected */
	stat = ntohl(*msg++);		/* reject_status */
	*msgp = msg;
	switch (stat) {
	case 0: return RPC_VERSMISMATCH;
	case 1: return RPC_AUTHERROR;
	}

	return RPC_FAILED;
}
