/*
 * xgetopt.c	extended getopt -- GNU getopt is not re-entrant, and doesn't
 *		have type conversions.
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdio.h>
#include <stdlib.h>
#include "nfsbench.h"
#include "xgetopt.h"

#define HELPINDENT	16

static void	print_help(char *cmd, struct xoption *opt);

int
xgetopt(int argc, char **argv, struct xoption *opt, int *last)
{
	struct xoption	*o;
	char		*arg;
	int		i, head;

	*last = i = 1;
	arg = argv[1];
	head = 1;
	while (i < argc) {
		*last = i;
		if (head && arg[0] == '-' && arg[1] == '-') {
			if (arg[2] == '\0')
				return 1;
			for (o = opt; o->type != OPT_END; o++) {
				if (o->lname && !strcmp(o->lname, arg+2))
					break;
			}
			if (o->type == OPT_END) {
				fprintf(stderr, "unknown option %s to %s\n",
					arg, argv[0]);
				return 0;
			}
			arg = argv[++i];
		} else {
			if (head && *arg++ != '-')
				return 1;
			if (arg[0] == '\0') {
				if (head) {
					fprintf(stderr,
						"missing option in `%s'\n",
						argv[i]);
					return 0;
				}
				arg = argv[++i];
				head = 1;
				continue;
			}
			for (o = opt; o->type != OPT_END; o++) {
				if (o->sname == arg[0])
					break;
			}
			if (o->type == OPT_END) {
				fprintf(stderr,
					"unknown option -%c to %s\n",
					arg[0], argv[0]);
				return 0;
			}
		}

		if (o->type == OPT_TRUE || o->type == OPT_FALSE) {
			*(int *) (o->data) = (o->type == OPT_TRUE);
			continue;
		}
		if (o->type == OPT_HELP) {
			print_help(argv[0], opt);
			return 0;
		}

		if (i >= argc || !head) {
			if (o->lname)
				fprintf(stderr,
					"missing argument to option %s\n",
					o->lname);
			else
				fprintf(stderr,
					"missing argument to option -%c\n",
					o->sname);
			return 0;
		}

		switch (o->type) {
		case OPT_INT:
			*(int *) (o->data) = atoi(arg);
			break;
		case OPT_STRING:
			*(char **) (o->data) = arg;
			break;
		case OPT_ADDR:
			if (!addr_parse(arg, (struct in_addr *) (o->data))) {
				fprintf(stderr, "%s: bad address\n", arg);
				return 0;
			}
			break;
		case OPT_PORT:
			*(unsigned short *) (o->data) = htons(atoi(arg));
			break;
		case OPT_FUNC:
			if (!opt->func(arg, o->data))
				return 0;
			break;
		case OPT_PTR:
			sscanf(arg, "%p", (void **) (o->data));
			break;
		}
		arg = argv[++i];
	}

	*last = i;
	return 1;
}

static void
print_help(char *cmd, struct xoption *opt)
{
	int	indent;

	fprintf(stderr, "%s options:\n", cmd);
	for (; opt->type != OPT_END; opt++) {
		indent = 4;
		if (opt->sname)
			fprintf(stderr, " -%c ", opt->sname);
		else
			fprintf(stderr, "    ");
		if (opt->lname) {
			fprintf(stderr, " --%s", opt->lname);
			indent += 3 + strlen(opt->lname);
		}
		if (!opt->help || opt->help[0] == '\0') {
			fprintf(stderr, "\n");
			continue;
		}
		if (indent > HELPINDENT) {
			fprintf(stderr, "\n");
			indent = 0;
		}
		fprintf(stderr, "%.*s %s\n",
			HELPINDENT - indent,
			"                                ",
			opt->help);
	}
}
